#include <Wire.h>
#include <LCD.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <LiquidCrystal_I2C.h>  // F Malpartida's NewLiquidCrystal library
#include "CyberLib.h"
#include <EEPROM.h>
#include <MsTimer2.h>

volatile uint8_t tic, Dimmer1, Dimm, Dimmer3; 

#define ONE_WIRE_BUS 8 //шина датчиков
/*-----( Declare Constants )-----*/
#define I2C_ADDR    0x20  // Define I2C Address for the PCF8574A 
//---(Following are the PCF8574 pin assignments to LCD connections )----
// This are different than earlier/different I2C LCD displays
#define BACKLIGHT_PIN  7
#define En_pin  4
#define Rw_pin  5
#define Rs_pin  6
#define D4_pin  0
#define D5_pin  1
#define D6_pin  2
#define D7_pin  3

#define  LED_OFF  1
#define  LED_ON  0



//***
byte butPin = 0;                 // вход кнопок
int val;                         // знач. АЦП при нажатой кнопке
byte butPush = 0;                // код нажатой кнопки
boolean butRel=1;                // отпущена ли сейчас кнопка
int butval[8]={0,0,0,0,0,0,0,0}; // АЦП кнопок
byte butsens=20;                 // чувствительность кнопок
boolean keysound=0;              // включен ли звук кнопок
byte buzzer = A3;                // выход динамика
byte menu=0;                     // текущий экран


float Tdef=45;               // аварийная температура Дефлегматора
float Tkub=99;               // аварийная температура Куба
float Tprotok=50;            // температура при которой включаем воду
float TempSTOP=78.2;         // температура при которой включается режим СТАРТ/СТОП
//float TempSTOP=25;         // температура при которой включается режим СТАРТ/СТОП

boolean ProcALARM=false;     //Авария датчика дефлегматора, по умолчанию фальсе
boolean ProcValveSTOP=false;     //Закрытия клапана обора при привышении температуры СТОП, по умолчанию фальсе

byte i;                      // счетчик
byte n;                      // счетчик
byte m=0;                    // счетчик
byte t=0;                    // счетчик
float T[]={0,0};             // переменная для хранения температуры
float Tp[]={0,0};            // переменная для хранения температуры
byte data[8];                // для хранения данных от датчика

byte TotalDS=0;   // число найденых датчиков на линии
byte addr[4][8] = {
   {0,0,0,0,0,0,0,0},
   {0,0,0,0,0,0,0,0},
   {0,0,0,0,0,0,0,0},
   {0,0,0,0,0,0,0,0},
}; 
OneWire  ds(8);            // на этот пин подключен DS
//-------
// Configuramos para comunicar con otros dispositivos 1-Wire
OneWire oneWire(ONE_WIRE_BUS);
 
// Indicamos el pin asignado al sensor 1-Wire a DallasTemperature
//DallasTemperature sensores(&oneWire);
//-------
//Клапана подачи воды,газа, отбора.

//const int otborPin =  4;         // номер выхода, подключенного к клапану
const int otborPin =  11;         // номер выхода, подключенного к клапану
long previousMillis = 0;          // храним время последнего переключения клапана
boolean in = false;
int interval;
//float val_0=0; // variable donde guardaremos la temperatura leida del sensor
//float val_1=0; // variable donde guardaremos la temperatura leida del sensor

/*-----( Declare objects )-----*/  
LiquidCrystal_I2C  lcd(I2C_ADDR,En_pin,Rw_pin,Rs_pin,D4_pin,D5_pin,D6_pin,D7_pin);

void setup()   /*----( SETUP: RUNS ONCE )----*/
{
  lcd.begin (16,2);  // initialize the lcd 
// Switch on the backlight
  lcd.setBacklightPin(BACKLIGHT_PIN,NEGATIVE);
  lcd.setBacklight(LED_ON);

//  sensores.begin();

  D3_Out; //Настраиваем порт на выход Dimmer1 
  D3_Low; //установить на выходах низкий уровень сигнала 
  D2_In; //настраиваем порт на вход для отслеживания прохождения сигнала через ноль   
    
  attachInterrupt(0, detect_up, LOW);  // настроить срабатывание прерывания int0 на pin 2  
  StartTimer1(halfcycle, 40);
  StopTimer1(); //остановить таймер
//  UART_Init(115200); //инициализация порта

  pinMode(otborPin, OUTPUT);  

// Процедура калибровки 
  
  if (analogRead(butPin) > 40) { // 40 - значение ненажатой кнопки
    lcd.backlight();  //Backlight ON if under program control
    lcd.print("KALIBROVKA");
    lcd.setCursor(0,1);
    delay(2000);
    for ( i = 1; i < 7; i++) {
    lcd.clear();
    
      switch (i) {
        case 1: // температуры
        lcd.setCursor(0,0); //Start at character 0 on line 0
        lcd.print("PRESS: UP");
        break;
        
        case 2: // температуры
        lcd.setCursor(0,0); //Start at character 0 on line 0
        lcd.print("PRESS: LEFT");
        break;
        
        case 3: // температуры
        lcd.setCursor(0,0); //Start at character 0 on line 0
        lcd.print("PRESS: OK");
        break;
        
        case 4: // температуры
        lcd.setCursor(0,0); //Start at character 0 on line 0
        lcd.print("PRESS: RIGHT");
        break;
        
        case 5: // температуры
        lcd.setCursor(0,0); //Start at character 0 on line 0
        lcd.print("PRESS: DOWN");
        break;
        
        case 6: // температуры
        lcd.setCursor(0,0); //Start at character 0 on line 0
        lcd.print("PRESS: CANCEL");
        break;        
      }
    while (analogRead(butPin) < 40) {};
    button ();
    butval[i]=val;
    lcd.clear();
    lcd.setCursor(0,1);
    lcd.print(i, DEC);
    lcd.print(" ");
    lcd.print(butval[i], DEC);
    EEPROM.write(224+i*2,butval[i]/4); // по 2 байта на значение начиная с 224
    EEPROM.write(225+i*2,butval[i]%4);
    delay(1000);
    }
  } 
// конец калибровки
// читаем АЦП кнопок
  else { 
    for ( i = 0; i < 8; i++) {
      butval[i] = EEPROM.read(224+i*2)*4+EEPROM.read(225+i*2);
    }
  }
//считываем адреса датчиков из памяти  
  read_DS();  
  keysound = EEPROM.read(241);
  butsens = EEPROM.read(242); if (butsens < 5 || butsens > 100) { butsens = 20;}  

//
    ds.reset();
    ds.write(0xCC,1);                                                            // пропуск ROM
    ds.write(0x44,1);                                                            // запускаем конвертацию
    MsTimer2::set(2000, Tizm);        // установка таймера на 2 сек. для изм. температуры 
    MsTimer2::start();                // запустить таймер
    
}// END Setup

void halfcycle()  //прерывания таймера 
{  
  tic++;  //счетчик   
  if(Dimmer1 < tic ) D3_High; //управляем выходом 
} 

void  detect_up()  // обработка внешнего прерывания. Сработает по переднему фронту синусоиды 
{   
 tic=0;             //обнулить счетчик 
 ResumeTimer1();
 attachInterrupt(0, detect_down, HIGH);  //перепрограммировать прерывание на другой обработчик 
}   

void  detect_down()  // обработка внешнего прерывания. Сработает по заднему фронту синусоиды 
{    

 StopTimer1();
 D3_Low; //логический ноль на выходы 
 tic=0;       //обнулить счетчик 
 attachInterrupt(0, detect_up, LOW); //перепрограммировать прерывание на другой обработчик 
}  
//************************************************************************* 
void Tizm() //обработка прерывания таймера - изм температуры
{
     for( n = 0; n < 2; n++) {
      ds.reset();
      ds.select(addr[n]);
      ds.write(0xBE);                                                            // считываем ОЗУ датчика
      for ( i = 0; i < 9; i++) {data[i] = ds.read();}                            // обрабатываем 9 байт
      Tp[n] = float((data[1] << 8) + data[0])/16;                                // высчитываем температуру :)
      if (Tp[n] == 85) {} else {T[n]=Tp[n];}                                     // это для того, чтобы при глюке датчика (обычно при переключениях 
      if (Tp[n] < 0) {tone(buzzer, 2000);} else {noTone(buzzer);}                // это для того, чтобы при глюке датчика (обычно при переключениях       
    }
    //для следующего запуска
    ds.reset();
    ds.write(0xCC,1);                                                            // пропуск ROM
    ds.write(0x44,1);                                                            // запускаем конвертацию 
}
void loop()   /*----( LOOP: RUNS OVER AND OVER AGAIN )----*/
{
    ds.reset();
    ds.write(0xCC,1);                                                            // пропуск ROM
    ds.write(0x44,1);                                                            // запускаем конвертацию
    for( n = 1; n < 25; n++) {                                                   //ждем 900 мс
      delay(5);            
      if (analogRead(butPin) > 40 && butRel) { Menu(button ()); }              //периодически проверяя не нажата ли кнопка
      if (analogRead(butPin) < 40) {butRel=1;}
    }   
      if (T[0]>=Tdef) 
      {
      ProcALARM=true;
      }
      if (ProcALARM==true)
      {
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("ALARM DEFLEGMATOR!!!");
      tone(buzzer, 1000, 500);   
      Dimmer1=255;  //255 отключить свет
      digitalWrite(otborPin, LOW); //Закрыть клапан отбора
      
      while (analogRead(butPin) > 1000) {};
      //прыгнуть на меню до запуска
      } else {
             Dimm = map(analogRead(10),0,1023,14,255);
             if (Dimmer1 < Dimm-1 || Dimmer1 > Dimm+1 ){
             Dimmer1=Dimm;
             }
        /////////////////////////
      if (T[1]>=TempSTOP){
      digitalWrite(otborPin, LOW); //Закрыть клапан отбора
      }else{
            //*****************Управление клапаном отбора*********************//
             unsigned long currentMillis = millis();
              if (in==false){interval=map(analogRead(1),0,1023,0,10000);}
              if(interval<=10) {digitalWrite(otborPin, LOW);}
              else if(interval>=10000) {digitalWrite(otborPin, HIGH);}
              else { if(currentMillis - previousMillis < interval) {digitalWrite(otborPin, HIGH); in=true;}
              else if(currentMillis - previousMillis >= interval && currentMillis - previousMillis < 9900 ) {digitalWrite(otborPin, LOW); in=true;}
              else {previousMillis = currentMillis; in=false;}}
            //****************************************************************// 
      }
Menu(0);      
      }                                       //проверка на аварию дефлегматора
    

} // END Loop

int button () {
    int exitMenu = 1;                                                                             // выход из меню
    butPush = 0;                                                                                  // код нажатой кнопки
    if (keysound) {tone(buzzer, 1000);}
    delay(15);                                                                                    // задержка для устранения дребезга контактов
    val = analogRead(butPin);                                                                     // читаем АЦП порта кнопок (значение от 0 до 1023)
    if (val < 21) {val = 30;}
    if (val > (butval[5]-butsens) && val < (butval[5]+butsens)) {butPush = 5;}  
    if (val > (butval[4]-butsens) && val < (butval[4]+butsens)) {butPush = 4;}
    if (val > (butval[3]-butsens) && val < (butval[3]+butsens)) {butPush = 3;}
    if (val > (butval[2]-butsens) && val < (butval[2]+butsens)) {butPush = 2;}
    if (val > (butval[1]-butsens) && val < (butval[1]+butsens)) {butPush = 1;}
    butRel=0;
    noTone(buzzer);
  return butPush;
}
//Считываем датчики из EEPROM
byte read_DS() {
  for (n=0; n<4; n++) {
    for (i=0; i<8; i++) {
      addr[n][i] = EEPROM.read(128+n*8+i);
    }
  }
  return 0;
}

//Запись датчиков в EEPROM
byte write_DS() {
  for (i=0; i<8; i++) {
    if (m==0) {EEPROM.write(128+t*8+i,0);}
    else {EEPROM.write(128+t*8+i,addr[m-1][i]);}
    }
  return 0;
}

void Menu(int butPush) {
    lcd.clear();
    switch (menu) {
      case 0: // температуры
      lcd.clear();
      //вывод температур и значений отбора и нагрева    
  lcd.home();
// Print our characters on the LCD
  lcd.backlight();  //Backlight ON if under program control
  lcd.setCursor(0,0); //Start at character 0 on line 0
  lcd.print("PW:");          
  lcd.print(Dimm);
  lcd.setCursor(0,1); //Start at character 0 on line 0
  lcd.print("OB:");  
  lcd.print(map(analogRead(1),0,1023,0,10000));
  //--вывод температуры
  lcd.setCursor(8,0); //Start at character 0 on line 0
  lcd.print("tD:");
  lcd.print(T[0]);
  lcd.setCursor(8,1); //Start at character 0 on line 0
  lcd.print("tK:");
  lcd.print(T[1]);     
        if (butPush == 1) {menu=2; Menu(0);}
        if (butPush == 5) {menu=2; Menu(0);}
      break;   
  
  // Меню настроек
        case 2: 
        lcd.setCursor(0,0); //Start at character 0 on line 0
        lcd.print("SETTINGS:");
        lcd.setCursor(0,1); //Start at character 0 on line 0
        lcd.print("PRESS OK");       
          if (butPush == 1) {menu=0; Menu(0);}
          if (butPush == 3) {menu=42; Menu(0);}
          if (butPush == 5) {menu=0; Menu(0);}
          if (butPush == 6) {menu=0; Menu(0);}
        break;
        
    // поиск датчиков 
      case 42: 
        i = 0;
        n=0;
        while (i != 1 )                                   // ищем датчики 
        {
          if ( ds.search(addr[n]) != 1) {
            ds.reset_search();
            i = 1; }
          else { 
            n++; 
            TotalDS = n;
          }
        }
        lcd.print("SEARCH:");
        lcd.setCursor(0,1);
        lcd.print(n, DEC);
        lcd.print("sarched");
        if (butPush == 1) {menu=101; Menu(0);}
        if (butPush == 4) {menu=100; Menu(0);}
        if (butPush == 5) {menu=101; Menu(0);}
      break;
        
   // меню настроек датчиков DS1820   
      case 100: 
        lcd.print("T");
        lcd.print(t+1,DEC);
        lcd.setCursor(0,1);
        if (m==0) {lcd.print("NOT FOUND!");}
           else {lcd.print(T[m-1],DEC);}
        if (butPush == 1) {if (t==0) {menu=101;} else {t--;} Menu(0);}
        if (butPush == 2) {if (m==0) {} else {m--;} Menu(0);}
        if (butPush == 3) {write_DS(); lcd.setCursor(0,1); lcd.print("WRITE TO EPPROM"); tone(buzzer, 1000, 500); menu=100; Menu(0);}
        if (butPush == 4) {m++; if (m>(TotalDS)) {m=TotalDS;} Menu(0);}
        if (butPush == 5) {if (t==1) {menu=101;} else {t++;}  Menu(0);}
      break; 
      
   // Выход из меню
      case 101: 
        lcd.print("EXIT MENU");
        lcd.setCursor(0,1);
        lcd.print(">OK TO EXIT!");
        if (butPush == 1) {t=1; menu=100; Menu(0);}
        if (butPush == 3) {menu=0; Menu(0);}
        if (butPush == 5) {t=0; menu=100; Menu(0);}  
        if (butPush == 6) {menu=0; Menu(0);}
      break;      
    }
}
